/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.wsdl.wsdl20.xerces;

import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.xerces.dom.DOMMessageFormatter;
import org.bluestemsoftware.specification.eoa.component.RootComponent.ComponentName;
import org.bluestemsoftware.specification.eoa.ext.feature.ws.transport.http.HTTPMethod;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.BindingElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.BindingFaultElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.BindingOperationElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ExtendedDocument;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ext.HTTPBindingElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ext.SOAPBindingElement;
import org.w3c.dom.DOMException;
import org.w3c.dom.Node;

public class BindingElementImpl extends AbstractWSDLElement implements BindingElement, SOAPBindingElement, HTTPBindingElement {

    private static final long serialVersionUID = 1L;

    private boolean isRead;
    private Map<QName, BindingFaultElement> faultElements = new HashMap<QName, BindingFaultElement>();;
    private Map<QName, BindingOperationElement> operationElements = new HashMap<QName, BindingOperationElement>();

    public BindingElementImpl(WSDL20DocumentImpl owner) {
        super(owner, NAME.getNamespaceURI(), NAME.getPrefix() + ":" + NAME.getLocalPart());
    }

    public BindingElementImpl() {
        super();
    }

    public boolean isRead() {
        return isRead;
    }

    public void setIsRead(boolean isRead) {
        this.isRead = isRead;
    }

    public void setName(String name) {
        setAttribute("name", name);
    }

    public String getName() {
        return getAttribute("name").equals("") ? null : getAttribute("name");
    }

    public void setInterface(ComponentName value) {
        if (value == null) {
            return;
        }
        String prefix = ((WSDL20DocumentImpl)getOwnerDocument()).addNamespace(value.getNamespaceURI());
        setAttribute("interface", prefix + ":" + value.getLocalPart());
    }

    public ComponentName getInterface() {
        String temp = getAttribute("interface");
        if (temp.equals("")) {
            return null;
        }
        return ComponentName.valueOf(this, temp);
    }

    public void setType(String value) {
        setAttribute("type", value);
    }

    public String getType() {
        return getAttribute("type").equals("") ? null : getAttribute("type");
    }

    public BindingFaultElement addFaultElement(BindingFaultElement faultElement) {
        if (operationElements.size() > 0) {
            super.insertBefore(faultElement, operationElements.values().iterator().next());
        } else {
            super.insertBefore(faultElement, null);
        }
        if (faultElement.getRef() == null) {
            throw new IllegalArgumentException("Required attribute 'ref' not set.");
        }
        faultElements.put(faultElement.getRef(), faultElement);
        return faultElement;
    }

    public Map<QName, BindingFaultElement> getFaultElements() {
        return faultElements;
    }

    public BindingOperationElement addOperationElement(BindingOperationElement operationElement) {
        super.insertBefore(operationElement, null);
        if (operationElement.getRef() == null) {
            throw new IllegalArgumentException("Required attribute 'ref' not set.");
        }
        operationElements.put(operationElement.getRef(), operationElement);
        return operationElement;
    }

    public Map<QName, BindingOperationElement> getOperationElements() {
        return operationElements;
    }

    public void setVersion(String value) {
        setAttributeNS(ExtendedDocument.WSDL_SOAP_NS, ExtendedDocument.WSDL_SOAP_NS_PREFIX + ":version", value);
    }

    public String getVersion() {
        return getAttributeNS(ExtendedDocument.WSDL_SOAP_NS, "version").equals("") ? null : getAttributeNS(
                ExtendedDocument.WSDL_SOAP_NS, "version");
    }

    public void setProtocol(String value) {
        setAttributeNS(ExtendedDocument.WSDL_SOAP_NS, ExtendedDocument.WSDL_SOAP_NS_PREFIX + ":protocol", value);
    }

    public String getProtocol() {
        return getAttributeNS(ExtendedDocument.WSDL_SOAP_NS, "protocol").equals("") ? null : getAttributeNS(
                ExtendedDocument.WSDL_SOAP_NS, "protocol");
    }

    public void setMEPDefault(String value) {
        setAttributeNS(ExtendedDocument.WSDL_SOAP_NS, ExtendedDocument.WSDL_SOAP_NS_PREFIX + ":mepDefault", value);
    }

    public String getMEPDefault() {
        return getAttributeNS(ExtendedDocument.WSDL_SOAP_NS, "mepDefault").equals("") ? null : getAttributeNS(
                ExtendedDocument.WSDL_SOAP_NS, "mepDefault");
    }

    public Boolean useHTTPCookies() {
        String temp = getAttributeNS(ExtendedDocument.WSDL_HTTP_NS, "cookies");
        if (temp.equals("")) {
            return null;
        }
        return Boolean.valueOf(temp);
    }

    public HTTPMethod getMethodDefault() {
        String temp = getAttributeNS(ExtendedDocument.WSDL_HTTP_NS_PREFIX, "methodDefault");
        if (temp.equals("")) {
            return null;
        } else {
            return HTTPMethod.valueOf(temp);
        }
    }

    public void setMethodDefault(HTTPMethod value) {
        if (value != null) {
            setAttributeNS(ExtendedDocument.WSDL_HTTP_NS, ExtendedDocument.WSDL_HTTP_NS_PREFIX + ":methodDefault",
                    value.toString());
        }
    }

    public void setUseHTTPCookies(Boolean value) {
        if (value != null) {
            setAttributeNS(ExtendedDocument.WSDL_HTTP_NS, ExtendedDocument.WSDL_HTTP_NS_PREFIX + ":cookies", value
                    .toString());
        }
    }

    public void setContentEncodingDefault(String value) {
        setAttributeNS(ExtendedDocument.WSDL_HTTP_NS, ExtendedDocument.WSDL_HTTP_NS_PREFIX
                + ":contentEncodingDefault", value);
    }

    public String getContentEncodingDefault() {
        return getAttributeNS(ExtendedDocument.WSDL_HTTP_NS_PREFIX, "contentEncodingDefault").equals("")
                ? null
                : getAttributeNS(ExtendedDocument.WSDL_HTTP_NS_PREFIX, "contentEncodingDefault");
    }

    public String getQueryParameterSeparatorDefault() {
        return getAttributeNS(ExtendedDocument.WSDL_HTTP_NS_PREFIX, "queryParameterSeparatorDefault").equals("")
                ? null
                : getAttributeNS(ExtendedDocument.WSDL_HTTP_NS_PREFIX, "queryParameterSeparatorDefault");
    }

    public void setQueryParameterSeparatorDefault(String value) {
        setAttributeNS(ExtendedDocument.WSDL_HTTP_NS, ExtendedDocument.WSDL_HTTP_NS_PREFIX
                + ":queryParameterSeparatorDefault", value);
    }

    @Override
    public Node appendChild(Node newChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(BindingFaultElement.NAME)) {
                return addFaultElement((BindingFaultElement)newChild);
            }
            if (elementName.equals(BindingOperationElement.NAME)) {
                return addOperationElement((BindingOperationElement)newChild);
            }
        }
        return super.insertBefore(newChild, null);
    }

    @Override
    public Node removeChild(Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(BindingFaultElement.NAME)) {
                faultElements.remove(((BindingFaultElement)oldChild).getRef());
            }
            if (elementName.equals(BindingOperationElement.NAME)) {
                operationElements.remove(((BindingOperationElement)oldChild).getRef());
            }
        }
        return super.removeChild(oldChild);
    }

    @Override
    public Node replaceChild(Node newChild, Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(BindingFaultElement.NAME) || elementName.equals(BindingOperationElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.replaceChild(newChild, oldChild);
    }

    @Override
    public Node insertBefore(Node newChild, Node refChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(BindingFaultElement.NAME) || elementName.equals(BindingOperationElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.insertBefore(newChild, refChild);
    }

}
