/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.wsdl.wsdl20.xerces;

import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.xerces.dom.DOMMessageFormatter;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.BindingOperationInputElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ExtendedDocument;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ext.HTTPBindingOperationInputElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ext.HTTPHeaderElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ext.SOAPBindingOperationInputElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ext.SOAPHeaderElement;
import org.w3c.dom.DOMException;
import org.w3c.dom.Node;

public class BindingOperationInputElementImpl extends AbstractWSDLElement implements BindingOperationInputElement, SOAPBindingOperationInputElement, HTTPBindingOperationInputElement {

    private static final long serialVersionUID = 1L;
    
    private Map<QName, SOAPHeaderElement> soapHeaderElements = new HashMap<QName, SOAPHeaderElement>();
    private Map<String, HTTPHeaderElement> httpHeaderElements = new HashMap<String, HTTPHeaderElement>();


    public BindingOperationInputElementImpl(WSDL20DocumentImpl owner) {
        super(owner, NAME.getNamespaceURI(), NAME.getPrefix() + ":" + NAME.getLocalPart());
    }

    public BindingOperationInputElementImpl() {
        super();
    }

    public void setMessageLabel(String value) {
        setAttribute("messageLabel", value);
    }

    public String getMessageLabel() {
        return getAttribute("messageLabel").equals("") ? null : getAttribute("messageLabel");
    }
    
    public void setContentEncoding(String value) {
        setAttributeNS(ExtendedDocument.WSDL_HTTP_NS, ExtendedDocument.WSDL_HTTP_NS_PREFIX + ":contentEncoding", value);
    }
    
    public String getContentEncoding() {
        return getAttributeNS(ExtendedDocument.WSDL_HTTP_NS, "contentEncodingDefault").equals("") ? null : getAttributeNS(ExtendedDocument.WSDL_HTTP_NS, "contentEncodingDefault");
    }
    
    public SOAPHeaderElement addSOAPHeaderElement(SOAPHeaderElement headerElement) {
        if (httpHeaderElements.size() > 0) {
            super.insertBefore(headerElement, httpHeaderElements.values().iterator().next());
        } else {
            super.insertBefore(headerElement, null);
        }
        if (headerElement.getElement() == null) {
            throw new IllegalArgumentException("Required attribute 'element' not set.");
        }
        soapHeaderElements.put(headerElement.getElement(), headerElement);
        return headerElement;
    }

    public Map<QName, SOAPHeaderElement> getSOAPHeaderElements() {
        return soapHeaderElements;
    }
    
    public HTTPHeaderElement addHTTPHeaderElement(HTTPHeaderElement headerElement) {
        if (headerElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        super.insertBefore(headerElement, null);
        httpHeaderElements.put(headerElement.getName(), headerElement);
        return headerElement;
    }

    public Map<String, HTTPHeaderElement> getHTTPHeaderElements() {
        return httpHeaderElements;
    }
    
    @Override
    public Node appendChild(Node newChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(SOAPHeaderElement.NAME)) {
                return addSOAPHeaderElement((SOAPHeaderElement)newChild);
            }
            if (elementName.equals(HTTPHeaderElement.NAME)) {
                return addHTTPHeaderElement((HTTPHeaderElement)newChild);
            }
        }
        return super.insertBefore(newChild, null);
    }
    
    @Override
    public Node removeChild(Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(SOAPHeaderElement.NAME)) {
                soapHeaderElements.remove(((SOAPHeaderElement)oldChild).getElement());
            }
            if (elementName.equals(HTTPHeaderElement.NAME)) {
                httpHeaderElements.remove(((HTTPHeaderElement)oldChild).getName());
            }
        }
        return super.removeChild(oldChild);
    }
    
    @Override
    public Node replaceChild(Node newChild, Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(SOAPHeaderElement.NAME) || elementName.equals(HTTPHeaderElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.replaceChild(newChild, oldChild);
    }

    @Override
    public Node insertBefore(Node newChild, Node refChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(SOAPHeaderElement.NAME) || elementName.equals(HTTPHeaderElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.insertBefore(newChild, refChild);
    }
    
}
