/**
 * Copyright 2008 Bluestem Software LLC.  All Rights Reserved.
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 */

package org.bluestemsoftware.open.eoa.ext.wsdl.wsdl20.xerces;

import java.util.HashMap;
import java.util.Map;

import javax.xml.namespace.QName;

import org.apache.xerces.dom.DOMMessageFormatter;
import org.bluestemsoftware.specification.eoa.component.RootComponent.ComponentName;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ServiceElement;
import org.bluestemsoftware.specification.eoa.ext.wsdl.wsdl20.ServiceEndpointElement;
import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;

public class ServiceElementImpl extends AbstractWSDLElement implements ServiceElement {

    private static final long serialVersionUID = 1L;

    private boolean isRead;
    private Map<String, ServiceEndpointElement> endpointElements = new HashMap<String, ServiceEndpointElement>();

    public ServiceElementImpl(WSDL20DocumentImpl owner) {
        super(owner, NAME.getNamespaceURI(), NAME.getPrefix() + ":" + NAME.getLocalPart());
    }
    
    public ServiceElementImpl() {
        super();
    }
    
    public boolean isRead() {
        return isRead;
    }

    public void setIsRead(boolean isRead) {
        this.isRead = isRead;
    }
    
    public void setName(String name) {
        setAttribute("name", name);
    }
    
    public String getName() {
        return getAttribute("name").equals("") ? null : getAttribute("name");
    }
    
    public void setInterface(ComponentName value) {
        if (value == null) {
            return;
        }
        String prefix = ((WSDL20DocumentImpl)getOwnerDocument()).addNamespace(value.getNamespaceURI());
        setAttribute("interface", prefix + ":" + value.getLocalPart());
    }
    
    public ComponentName getInterface() {
        String temp = getAttribute("interface");
        if (temp.equals("")) {
            return null;
        }
        return ComponentName.valueOf(this, temp);
    }
    
    public ServiceEndpointElement addEndpointElement(ServiceEndpointElement endpointElement) {
        if (endpointElement.getName() == null) {
            throw new IllegalArgumentException("Required attribute 'name' not set.");
        }
        endpointElements.put(endpointElement.getName(), endpointElement);
        super.insertBefore(endpointElement, null);
        return endpointElement;
    }

    public Map<String, ServiceEndpointElement> getEndpointElements() {
        return endpointElements;
    }
    
    @Override
    public Node appendChild(Node newChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(ServiceEndpointElement.NAME)) {
                return addEndpointElement((ServiceEndpointElement)newChild);
            }
        }
        return super.insertBefore(newChild, null);
    }
    
    @Override
    public Node removeChild(Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(ServiceEndpointElement.NAME)) {
                endpointElements.remove(((Element)oldChild).getAttribute("name"));
            }
        }
        return super.removeChild(oldChild);
    }
    
    @Override
    public Node replaceChild(Node newChild, Node oldChild) throws DOMException {
        if (oldChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = oldChild.getNamespaceURI();
            String localPart = oldChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(ServiceEndpointElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.replaceChild(newChild, oldChild);
    }

    @Override
    public Node insertBefore(Node newChild, Node refChild) throws DOMException {
        if (newChild.getNodeType() == Node.ELEMENT_NODE) {
            String namespaceURI = newChild.getNamespaceURI();
            String localPart = newChild.getLocalName();
            QName elementName = new QName(namespaceURI, localPart);
            if (elementName.equals(ServiceEndpointElement.NAME)) {
                throw new DOMException(DOMException.HIERARCHY_REQUEST_ERR, DOMMessageFormatter.formatMessage(
                        DOMMessageFormatter.DOM_DOMAIN, "HIERARCHY_REQUEST_ERR", null));
            }
        }
        return super.insertBefore(newChild, refChild);
    }
}
